#include "config.h"
#include <stdio.h>
#include <syslog.h>

#define	DIALER_SUCCESS			0
#define	DIALER_INVALID_PARAMETER	1
#define	DIALER_PROGRAM_FAILED		2
#define	DIALER_TIMEOUT_ERROR		3

static char *input(int timeout);
static void empty(int timeout);

static void empty(int timeout)
{
	char *ret;

	for(;;)
	{
		ret = input(timeout);
		if(!*ret)
			return;
	}
}

static char *input(int timeout)
{
	static char buf[256];

	int bp = 0;
	struct timeval tv;
	fd_set inp;

	tv.tv_usec = 0;
	tv.tv_sec = timeout;

	while(bp < 256)
	{
		FD_SET(0, &inp);
		if(select(1, &inp, NULL, NULL, &tv) < 1)
			break;

		tv.tv_usec = 500000;
		tv.tv_sec = 0;
		if(read(0, buf + bp, 1) < 1)
			break;

		if(!bp)
		{
			if(*buf == '\r' || *buf == '\n')
			{
				tv.tv_usec = 0;
				tv.tv_sec = timeout;
				continue;
			}
			if(isspace(*buf))
				continue;
		}
		if(buf[bp] == '\r')
			break;

		++bp;
	}
	buf[bp--] = 0;
	while(isspace(buf[bp]) && bp > 0)
		buf[bp--] = 0;
	++bp;
	/* syslog(LOG_ERR, "input <%s>\n", buf); */
	return buf;
}

int main(int argc, char **argv)
{
	int nextnum;
	char *ret;
	char *uid, *pwd, *cmd;
	FILE *fp = fopen("/etc/ppp/dial-secrets", "r");
	char lbuf[256];

	openlog("pppdial", LOG_PID, LOG_DAEMON);
	if(argc < 2)
	{
		syslog(LOG_CRIT, "no phone numbers specified");
		exit(DIALER_INVALID_PARAMETER);
	}
	
	if(!fp)
	{
		syslog(LOG_CRIT, "no dial secrets");
		exit(DIALER_PROGRAM_FAILED);
	}
			
	syslog(LOG_INFO, "reset modem");
	printf("\r");
	fflush(stdout);
	empty(1);
	printf("ATZ\r");
	fflush(stdout);

	for(;;)
	{
		ret = input(3);
		if(!*ret)
		{
			syslog(LOG_ERR, "reset failed");		
			exit(DIALER_TIMEOUT_ERROR);
		}
		if(!strcmp(ret, "OK"))
			break;
	}

	while(--argc)
	{
		empty(1);
		++argv;
		nextnum = 0;
		syslog(LOG_INFO, "calling %s", *argv);
		rewind(fp);
		for(;;)
		{
			fgets(lbuf, 256, fp);
			if(feof(fp))
			{
				syslog(LOG_ERR, "%s: no secret", *argv);
				exit(DIALER_PROGRAM_FAILED);
			}
			if(!strcmp(strtok(lbuf, " \t\n:,;"), *argv))
				break;
		}
		uid = strtok(NULL, " \t\n:,;");
		pwd = strtok(NULL, " \t\n:,;");
		cmd = strtok(NULL, " \t\n:,;");
		printf("ATD%s\r", *argv);
		fflush(stdout);
		for(;;)
		{
			ret = input(60);
			if(!*ret)
			{			
				syslog(LOG_ERR, "call timed out");
				exit(DIALER_TIMEOUT_ERROR);
			}
			if(!strncmp(ret, "CONNECT", 7))
				break;
			if(!strncmp(ret, "NO ", 3))
			{
				++nextnum;
				syslog(LOG_ERR, "%s: failed", *argv);
				break;
			}
			if(!strncmp(ret, "BUSY", 4))
			{
				++nextnum;
				syslog(LOG_ERR, "%s: busy", *argv);
				break;
			}
		}
		if(nextnum)
			continue; 
		
		while(uid)
		{
			ret = input(5);
			if(!*ret)
			{
				if(!nextnum)
				{
					++nextnum;
					printf("\r");
					fflush(stdout);
				}
				else
				{
					syslog(LOG_ERR, "%s: no login", *argv);
					fputs("+++", stdout);
					sleep(2);
					printf("ATH\r");
					fflush(stdout);
					break;
				}
			}
			nextnum = 0;
			if(!strnicmp(ret, "user", 4) || !strnicmp(ret, "login", 5))
				break;
		}
		if(nextnum)
			continue;

		if(uid)
		{
			printf("%s\r", uid);
			fflush(stdout);
		}
		while(pwd)
		{
			ret = input(5);
			if(!*ret)
			{
				syslog(LOG_ERR, "%s: no passwd", *argv);
				fputs("+++", stdout);
				sleep(2);
				printf("ATH\r");
				fflush(stdout);
				++nextnum;
				break;
			}
			if(!strnicmp(ret, "pass", 4))
				break;
		}
		if(nextnum)
			continue;

		if(pwd)
		{
			printf("%s\r", pwd);
			fflush(stdout);
		}

		/* if we have a command, wait for end at prompt */

		while(cmd)
		{
			ret = input(3);
			if(!*ret)
			{
				printf("%s\r", cmd);
				fflush(stdout);
				break;
			}
		}
		return DIALER_SUCCESS;
	}
	fclose(fp);
	return DIALER_TIMEOUT_ERROR;
}


